@@ -118,7 +118,7 @@ struct SLConn {
 /* This object is a singleton that keeps track of all data loggers.
 */
 static struct SLGlobal {
-  /* Protected by MUTEX_STATIC_MASTER */
+  /* Protected by MUTEX_STATIC_MAIN */
   sqlite3_mutex *mutex;           /* Recursive mutex */
   int nConn;                      /* Size of aConn[] array */
 
@@ -313,7 +313,7 @@ static void sqllogCopydb(struct SLConn *p, const char *zSearch, int bLog){
 
       /* Generate a file-name to use for the copy of this database */
       iDb = sqllogglobal.iNextDb++;
-      zInit = sqlite3_mprintf("%s_%d.db", sqllogglobal.zPrefix, iDb);
+      zInit = sqlite3_mprintf("%s_%02d.db", sqllogglobal.zPrefix, iDb);
 
       /* Create the backup */
       assert( sqllogglobal.bRec==0 );
@@ -376,7 +376,7 @@ static void sqllogOpenlog(struct SLConn *p){
       char *zVar = getenv(ENVIRONMENT_VARIABLE1_NAME);
       if( zVar==0 || strlen(zVar)+10>=(sizeof(sqllogglobal.zPrefix)) ) return;
       sqlite3_snprintf(sizeof(sqllogglobal.zPrefix), sqllogglobal.zPrefix,
-                        "%s/sqllog_%d", zVar, getProcessId());
+                        "%s/sqllog_%05d", zVar, getProcessId());
       sqlite3_snprintf(sizeof(sqllogglobal.zIdx), sqllogglobal.zIdx,
                         "%s.idx", sqllogglobal.zPrefix);
       if( getenv(ENVIRONMENT_VARIABLE2_NAME) ){
@@ -387,7 +387,7 @@ static void sqllogOpenlog(struct SLConn *p){
     }
 
     /* Open the log file */
-    zLog = sqlite3_mprintf("%s_%d.sql", sqllogglobal.zPrefix, p->iLog);
+    zLog = sqlite3_mprintf("%s_%05d.sql", sqllogglobal.zPrefix, p->iLog);
     p->fd = fopen(zLog, "w");
     sqlite3_free(zLog);
     if( p->fd==0 ){
@@ -467,28 +467,28 @@ static int sqllogTraceDb(sqlite3 *db){
 */
 static void testSqllog(void *pCtx, sqlite3 *db, const char *zSql, int eType){
   struct SLConn *p = 0;
-  sqlite3_mutex *master = sqlite3_mutex_alloc(SQLITE_MUTEX_STATIC_MASTER);
+  sqlite3_mutex *mainmtx = sqlite3_mutex_alloc(SQLITE_MUTEX_STATIC_MAIN);
 
   assert( eType==0 || eType==1 || eType==2 );
   assert( (eType==2)==(zSql==0) );
 
   /* This is a database open command. */
   if( eType==0 ){
-    sqlite3_mutex_enter(master);
+    sqlite3_mutex_enter(mainmtx);
     if( sqllogglobal.mutex==0 ){
       sqllogglobal.mutex = sqlite3_mutex_alloc(SQLITE_MUTEX_RECURSIVE);
     }
-    sqlite3_mutex_leave(master);
+    sqlite3_mutex_leave(mainmtx);
 
     sqlite3_mutex_enter(sqllogglobal.mutex);
     if( sqllogglobal.bRec==0 && sqllogTraceDb(db) ){
 
-      sqlite3_mutex_enter(master);
+      sqlite3_mutex_enter(mainmtx);
       p = &sqllogglobal.aConn[sqllogglobal.nConn++];
       p->fd = 0;
       p->db = db;
       p->iLog = sqllogglobal.iNextLog++;
-      sqlite3_mutex_leave(master);
+      sqlite3_mutex_leave(mainmtx);
 
       /* Open the log and take a copy of the main database file */
       sqllogOpenlog(p);
@@ -507,7 +507,7 @@ static void testSqllog(void *pCtx, sqlite3 *db, const char *zSql, int eType){
 
     /* A database handle close command */
     if( eType==2 ){
-      sqlite3_mutex_enter(master);
+      sqlite3_mutex_enter(mainmtx);
       if( i<sqllogglobal.nConn ){
         if( p->fd ) fclose(p->fd);
         p->db = 0;
@@ -524,7 +524,7 @@ static void testSqllog(void *pCtx, sqlite3 *db, const char *zSql, int eType){
           memmove(p, &p[1], nShift*sizeof(struct SLConn));
         }
       }
-      sqlite3_mutex_leave(master);
+      sqlite3_mutex_leave(mainmtx);
 
     /* An ordinary SQL command. */
     }else if( i<sqllogglobal.nConn && p->fd ){
